{% load i18n %}
{% blocktrans %}
Hello <strong>{{ username }}</strong>
<br>
<br>
Your file is ready : <a href="{{ url }}">Download</a>.
<br>
If you can't click on this link, copy-paste the following text into your browser : {{ url }}
{% endblocktrans %}
