# Generated by Django 2.0.9 on 2018-10-05 13:02

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('geostore', '0014_auto_20180918_1210'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='layer',
            options={'permissions': (('can_update_features_properties', 'Is able update geometries properties'),)},
        ),
    ]
