# Generated by Django 3.0.6 on 2020-06-25 12:44

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('geostore', '0037_auto_20200624_1440'),
    ]

    operations = [
        migrations.AlterField(
            model_name='feature',
            name='layer',
            field=models.ForeignKey(db_index=False, on_delete=django.db.models.deletion.PROTECT, related_name='features', to='geostore.Layer'),
        ),
    ]
