# Generated by Django 2.0.13 on 2019-05-23 13:28

from django.db import migrations, models
import geostore.models


class Migration(migrations.Migration):

    dependencies = [
        ('geostore', '0023_layer_type_geom'),
    ]

    operations = [
        migrations.AlterField(
            model_name='layer',
            name='geom_type',
            field=models.IntegerField(choices=[(0, geostore.GeometryTypes(0)), (1, geostore.GeometryTypes(1)), (3, geostore.GeometryTypes(3)), (4, geostore.GeometryTypes(4)), (5, geostore.GeometryTypes(5)), (6, geostore.GeometryTypes(6)), (7, geostore.GeometryTypes(7))], null=True),
        ),
    ]
