# Generated by Django 2.2.5 on 2019-09-26 08:03

import django.contrib.postgres.indexes
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geostore', '0028_remove_layer_group'),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='feature',
            name='geo_featu_identif_212b28_idx',
        ),
        migrations.RemoveIndex(
            model_name='feature',
            name='geo_featu_layer_i_3dcdde_gist',
        ),
        migrations.RemoveIndex(
            model_name='feature',
            name='geo_feat_layer_i_921f95_idx',
        ),
        migrations.RemoveIndex(
            model_name='feature',
            name='geo_featu_updated_62a19a_idx',
        ),
        migrations.RemoveIndex(
            model_name='feature',
            name='geo_featu_updated_cfeac3_idx',
        ),
        migrations.AddIndex(
            model_name='feature',
            index=models.Index(fields=['layer'], name='geostore_fe_layer_i_71cad6_idx'),
        ),
        migrations.AddIndex(
            model_name='feature',
            index=models.Index(fields=['updated_at'], name='geostore_fe_updated_fcdc6b_idx'),
        ),
        migrations.AddIndex(
            model_name='feature',
            index=models.Index(fields=['updated_at', 'layer'], name='geostore_fe_updated_bdb823_idx'),
        ),
        migrations.AddIndex(
            model_name='feature',
            index=models.Index(fields=['identifier'], name='geostore_fe_identif_f6c552_idx'),
        ),
        migrations.AddIndex(
            model_name='feature',
            index=django.contrib.postgres.indexes.GistIndex(fields=['layer', 'geom'], name='geostore_fe_layer_i_c3168f_gist'),
        ),
    ]
