# Generated by Django 3.1.1 on 2020-10-08 12:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geostore', '0041_auto_20200625_1515'),
    ]

    operations = [
        migrations.AddField(
            model_name='layer',
            name='routable',
            field=models.BooleanField(default=False, help_text='Used for make layer routable'),
        ),
    ]
